#define STB_IMAGE_IMPLEMENTATION
#define STB_IMAGE_WRITE_IMPLEMENTATION
#include "stb_image.h"
#include "stb_image_write.h"

#include<stdio.h>
#include<algorithm>
#include<string>
#include<vector>
#include<math.h>
using namespace std;

class Image {
public:
	int width;
	int height;
	unsigned char* data;
	~Image() { if (data) delete[] data; }
	Image() {data = NULL;}
	Image(int w, int h) {
		width = w;
		height = h;
		data = new unsigned char[width * height * 4];
		clear();
	}
	Image(string filename) {
		data = NULL;
		Load(filename);
	}
	void clear(int a=0, int r=0, int g=0, int b=0) {
		for (int id = 0; id < width * height; id++) {
			data[id * 4 + 0] = r;
			data[id * 4 + 1] = g;
			data[id * 4 + 2] = b;
			data[id * 4 + 3] = a;
		}
	}
	void setcolor(int i, int j, int c, int x) {
		int id = i * width + j;
		if (x < 0)x = 0; if (x > 255)x = 255;
		data[id * 4 + c] = x;
	}
	int getcolor(int i, int j, int c) {
		int id = i * width + j;
		return data[id * 4 + c];
	}

	void Load(string filename) {
		FILE* fp = fopen(filename.c_str(), "rb");
		if (!fp) { printf("ERROR : %s is not exist\n", filename.c_str()); return; }
		int cc;
		if (data) delete[] data;
		data = stbi_load_from_file(fp, &width, &height, &cc, 4);
		fclose(fp);
	}
	void Save(string filename) {
		if (data) {
			stbi_write_png(filename.c_str(), width, height, 4, data, 4 * width);
			printf("%s Save success!\n", filename.c_str());
		}
	}
};
struct vec3 {
	double x, y, z;
	vec3() { x = y = z = 0; }
	vec3(double xx, double yy, double zz) { x = xx; y = yy; z = zz; }
	double dist() { return sqrt(x * x + y * y + z * z); }
	void normalize() {
		double dd = dist();
		x /= dd;
		y /= dd;
		z /= dd;
	}
};
void wahaha(Image& in, Image& img, double radius, double border = 0.1) {
	int i, j, k, l;
	int w, h;
	w = in.width;
	h = in.height;
	vec3 dir(3, 4, 1);
	dir.normalize();
	for (i = 0; i < h; i++) {
		for (j = 0; j < w; j++) {
			int cnt = 0;
			vec3 color;
			vec3 ix;
			double weight = 0;
			for (k = -radius - 1; k <= radius + 1; k++) {
				for (l = -radius - 1; l <= radius + 1; l++) {
					int ti = i + k;
					int tj = j + l;
					if (ti < 0)ti = 0; if (ti >= h)ti = h-1;
					if (tj < 0)tj = 0; if (tj >= w)tj = w - 1;
					vec3 kl(k, l, 0);
					double cw = kl.dist();
					if (cw < radius)
					{
						vec3 rr;
						rr.x = in.getcolor(ti, tj, 0) / 255.0;
						rr.y = in.getcolor(ti, tj, 1) / 255.0;
						rr.z = in.getcolor(ti, tj, 2) / 255.0;
								
						if (rr.dist() > 0.01) {
							double ww = 1.0 / (cw + 1);
							color.x += rr.x * ww;
							color.y += rr.y * ww;
							color.z += rr.z * ww;

							vec3 cur((double)k / radius, (double)l / radius, 0);
							if (cur.dist() > 1.0)
								cur.normalize();
							ix.x += cur.x * ww;
							ix.y += cur.y * ww;

							weight += ww;

							cnt++;
						}
					}
				}
			}

			if (cnt > 0) {
				color.x /= weight;
				color.y /= weight;
				color.z /= weight;
				ix.x /= weight;
				ix.y /= weight;
				ix.z = 0;
				ix.z = 1.0 - ix.dist();

				if (ix.z < border) { color = vec3(1.0, 1.0, 1.0); }

				double diffuse = ix.x * dir.x + ix.y * dir.y + ix.z * dir.z;
				if (diffuse < -0.2) diffuse = -0.2;

				color.x += diffuse;
				color.y += diffuse;
				color.z += diffuse;

				int r, g, b;
				r = color.x * 255;
				g = color.y * 255;
				b = color.z * 255;
				img.setcolor(i, j, 0, r);
				img.setcolor(i, j, 1, g);
				img.setcolor(i, j, 2, b);
				img.setcolor(i, j, 3, 255);
			}
		}
	}
}
int main() {
	Image input("input.png");
	Image img(input.width, input.height);
	wahaha(input, img, 7, 0.2);
	img.Save("output.png");
}